@@ -1,10 +1,20 @@
 import abc
 import inspect
 import logging
+import sys
 
 from .resolvable import Resolvable, Promise
 from .util import multihash, Comparable, unquote
 
+# Define the type for `A | B` expressions.
+if sys.version_info >= (3, 10):  # pragma: specific no cover 3.7 3.8 3.9
+    from types import UnionType
+else:  # pragma: specific no cover 3.10
+
+    class UnionType:
+        pass
+
+
 __all__ = [
     "TypeMeta",
     "VarArgs",
@@ -75,6 +85,8 @@ class VarArgs(AbstractType):
             number of types. Defaults to `object`.
     """
 
+    __slots__ = ["type"]
+
     def __init__(self, type=object):
         self.type = ptype(type)
 
@@ -159,6 +171,8 @@ class Union(ComparableType):
         alias (str, optional): Give the union a name.
     """
 
+    __slots__ = ["_types"]
+
     def __init__(self, *types, alias=None):
         # Lazily convert to a set to avoid resolution errors.
         self._types = tuple(ptype(t) for t in types)
@@ -220,6 +234,8 @@ class Type(ComparableType):
         type (type): Type to encapsulate.
     """
 
+    __slots__ = ["_type"]
+
     def __init__(self, type):
         self._type = type
 
@@ -410,6 +426,10 @@ def ptype(obj):
                 f"Please open an issue at https://github.com/wesselb/plum/issues"
             )  # pragma: no cover
 
+    # Support Python 3.10 `int | float` syntax.
+    elif isinstance(obj, UnionType):  # pragma: specific no cover 3.7 3.8 3.9
+        return Union(*(ptype(t) for t in obj.__args__))
+
     # Strings are forward references.
     if isinstance(obj, str):
         return get_forward_reference(obj)
